/********************************** <lalVerbatim file="COHInitSearchCV">
Author: Julien Sylvestre
$Id: InitSearch.c,v 1.4 2002/12/02 17:42:31 julien Exp $
**************************************************** </lalVerbatim> */

#include <lal/LALStdlib.h>
#include <lal/TFClusters.h>
#include <lal/AVFactories.h>
#include <InitSearch.h>
#include <unistd.h>
#include <lal/DetectorSite.h>
#include <lal/LALConstants.h>
#include <strings.h>

NRCSID( INITSEARCHC, "tfclusters $Id: InitSearch.c,v 1.4 2002/12/02 17:42:31 julien Exp $");


/* PARAMETERS:
   argv[0] = "-filterparams"
   [1] = channel name; if SNGL_BURST, first two letters go into IFO. If GDS_TRIGGER, full name goes into subtype.
   [2] = table (SNGL_BURST or GDS_TRIGGER)
   [3] = total number of data
   [4] = time resolution in seconds
   [5] = sampling frequency (Hz)
   [6] = black pixel probability
   [7] = absolute error goal on power threshld. Set to <0 for white noise of unit variance at input. Set to 0 exactly and pass a spectrum to bypass Rice estimation. If a spectrum is passed, it must be from DC to Nyquist inclusively, and have a resolution 1/argv[5]. It should use a rectangular window, or else bpp will be different from argv[8]. A spectrum must be passed if a transfer function is passed.
   [8] = alpha
   [9] = minimum frequency to process (Hz)
   [10] = maximum frequency to process (Hz)
   [11] = number of standard deviations above estimated mean to reject in second pass at mean and variance estimation
   [12] = sigma
   [13] = distance for s1 = 1, s2 = 1
   [14] = distance for s1 = 1, s2 = 2
   ...
   [.] = distance for s1 = 1, s2 = sigma - 1
   [..] = distance for s1 = 2, s2 = 2
   ...
   [13+sigma*(sigma-1)/2] = distance for s1 = sigma-1, s2 = sigma-2
   [14+sigma*(sigma-1)/2] : if argument == 1, writes pixel coordinates to database table sngl_transdata

   fInd = 15+sigma*(sigma-1)/2 below

   [fInd] = number of interferometers (NIFO)
   [fInd + 1] = name of IFO #1 (LHO2k, LHO4k, LLO4k, VIRGO, GEO, TAMA)
   ...
   [fInd + NIFO] = name of last IFO
   [fInd + NIFO + 1] = min alpha
   [fInd + NIFO + 2] = max alpha
   [fInd + NIFO + 3] = number of points in [min alpha, max alpha]
   [fInd + NIFO + 4] = min delta
   [fInd + NIFO + 5] = max delta
   [fInd + NIFO + 6] = number of points in [min delta, max delta] (uniform in sin)
   [fInd + NIFO + 7] = min psi
   [fInd + NIFO + 8] = max psi
   [fInd + NIFO + 9] = number of points in [min psi, max psi]
   [fInd + NIFO + 10] = min plusDotcross 
   [fInd + NIFO + 11] = max plusDotcross
   [fInd + NIFO + 12] = number of points (uniform in sin)
   [fInd + NIFO + 13] = min plus2cross
   [fInd + NIFO + 14] = max plus2cross
   [fInd + NIFO + 15] = number of points (uniform in log)
   [fInd + NIFO + 16] = central frequency (Hz) of rectangle in tf-plane to consider
   [fInd + NIFO + 17] = bandwidth of rectangle
   [fInd + NIFO + 18] = min time (s)
   [fInd + NIFO + 19] = max time (s)
   [fInd + NIFO + 20] = number of times to apply pre-processing filters
   [fInd + NIFO + 21] = & 1: TFC on +; & 2: TFC on x; & 4: max on +; & 8: max on x
*/

void LALInitSearch(
    LALStatus             *status,
    void                 **searchParams,
    LALInitSearchParams   *initSearchParams
    )
{

  CohereSearchParams *params;
  REAL8 T, alpha=0, minf, maxf, cmint, cmaxt;
  INT4 sigma, tmpi4;
  UINT4 i, delL, fInd, tmpU4;
  INT4 fs, NData;
  UINT4 *delta;
  REAL4 bpp, eGoal, Nsigma, ccf, cbw;
  char *name;
  DbTable table;
  BOOLEAN wBlobs = 0;

  /*
  printf("%g\n",cosh(acosh(3.02993)/3.0));
  */

  INITSTATUS (status, "LALInitSearch", INITSEARCHC);
  ATTATCHSTATUSPTR (status);

  /*
  LALCoherentEstimation(status->statusPtr, NULL, NULL, NULL,cosh,acosh);
  CHECKSTATUSPTR (status);
  */

  /*
   *
   * Sanity checks on the input, output, params
   *
   */

  /*
  printf("Sleeping\n");
  usleep(10000000);
  printf("Waking up\n");
  */

  if ( !(initSearchParams) ){
    ABORT( status, INITSEARCHH_ENULL, INITSEARCHH_MSGENULL);
  }
  if ( (*searchParams) ){
    ABORT( status, INITSEARCHH_ENNUL, INITSEARCHH_MSGENNUL);
  }

  if ( !(initSearchParams->argv) ){
    ABORT( status, INITSEARCHH_ENULL, INITSEARCHH_MSGENULL);
  }

  
  /************* TFCLUSTERS Parameters ****************/

  if ( initSearchParams->argc < 13){
        ABORT( status, INITSEARCHH_EARGS, INITSEARCHH_MSGEARGS ); 
  }
 
  if (strcmp( initSearchParams->argv[0], "-filterparams" )){
    ABORT( status, INITSEARCHH_EARGS, INITSEARCHH_MSGEARGS);
  }

  name = initSearchParams->argv[1];
  if(!strlen(name)) {
    ABORT ( status, INITSEARCHH_ENAM, INITSEARCHH_MSGENAM );
  }

  if(!strcmp(initSearchParams->argv[2], "SNGL_BURST")) {
    table = SNGL_BURST;
  }
  else {
    if(!strcmp(initSearchParams->argv[2], "GDS_TRIGGER")) {
      table = GDS_TRIGGER;
    }
    else {
      ABORT ( status, INITSEARCHH_ETAB, INITSEARCHH_MSGETAB );
    }
  }

  NData = atoi(initSearchParams->argv[3]);
  if ( NData <= 0 ) {
    ABORT ( status, INITSEARCHH_ESPOS, INITSEARCHH_MSGESPOS );
  }


  T = atof(initSearchParams->argv[4]);
  if ( isnan(T) ) {
    ABORT ( status, INITSEARCHH_ENAN, INITSEARCHH_MSGENAN );
  }
  if ( T <= 0 ) {
    ABORT ( status, INITSEARCHH_ESPOS, INITSEARCHH_MSGESPOS );
  }

  fs = atoi(initSearchParams->argv[5]);
  if ( fs <= 0 ) {
    ABORT ( status, INITSEARCHH_ESPOS, INITSEARCHH_MSGESPOS );
  }

  bpp = atof(initSearchParams->argv[6]);
  if ( isnan(bpp) ) {
    ABORT ( status, INITSEARCHH_ENAN, INITSEARCHH_MSGENAN );
  }
  if ( bpp < 0.0 || bpp > 1.0 ) {
    ABORT ( status, INITSEARCHH_EAOR, INITSEARCHH_MSGEAOR );
  }

  eGoal = atof(initSearchParams->argv[7]);
  if ( isnan(eGoal) ) {
    ABORT ( status, INITSEARCHH_ENAN, INITSEARCHH_MSGENAN );
  }

  alpha = atof(initSearchParams->argv[8]);
  if ( isnan(alpha) ) {
    ABORT ( status, INITSEARCHH_ENAN, INITSEARCHH_MSGENAN );
  }
  /*
  if ( alpha < 0.0 || alpha > 1.0 ) {
    ABORT ( status, INITSEARCHH_EAOR, INITSEARCHH_MSGEAOR );
  }
  */

  minf = atof(initSearchParams->argv[9]);
  if ( isnan(minf) ) {
    ABORT ( status, INITSEARCHH_ENAN, INITSEARCHH_MSGENAN );
  }
  if ( minf < 0.0 ) {
    ABORT ( status, INITSEARCHH_EPOS, INITSEARCHH_MSGEPOS );
  }

  maxf = atof(initSearchParams->argv[10]);
  if ( isnan(maxf) ) {
    ABORT ( status, INITSEARCHH_ENAN, INITSEARCHH_MSGENAN );
  }
  if ( maxf < 0.0 ) {
    ABORT ( status, INITSEARCHH_EPOS, INITSEARCHH_MSGEPOS );
  }

  if( minf > maxf ) {
    ABORT ( status, INITSEARCHH_EMMF, INITSEARCHH_MSGEMMF );
  }

  Nsigma = atof(initSearchParams->argv[11]);
  if(Nsigma <= 0) {
    ABORT ( status, INITSEARCHH_ESPOS, INITSEARCHH_MSGESPOS );
  }
  
  sigma = atoi(initSearchParams->argv[12]);
  if(sigma<=0) {
    ABORT ( status, INITSEARCHH_ESPOS, INITSEARCHH_MSGESPOS );
  }

  delL = sigma * (sigma - 1) / 2;

  if(initSearchParams->argc < 14 + delL) {
      ABORT( status, INITSEARCHH_EARGS, INITSEARCHH_MSGEARGS );
  }

  if(sigma > 1) {

    delta = (UINT4 *) LALMalloc ( delL * sizeof(UINT4) );
    if(!delta) {
      ABORT (status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
    }

    for(i=0; i<delL; i++) {
    
      tmpi4 = atoi(initSearchParams->argv[13+i]);
      if(tmpi4 < 0) {
	ABORT ( status, INITSEARCHH_EPOS, INITSEARCHH_MSGEPOS );
      }

      delta[i] = (UINT4)tmpi4;
    }
  }
  else delta = NULL;

  if(strcmp(initSearchParams->argv[13 + delL],"1")) {
    wBlobs = 1;
  } else {
    wBlobs = 0;
  }

  /*
   *
   * allocate memory 
   *
   */

  *searchParams = params = LALMalloc (sizeof( CohereSearchParams )); 
  if ( !params )
  {
    ABORT (status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
  }


  params->dir = (CListDir*)LALMalloc(sizeof(CListDir));
  if(!params->dir) {
    ABORT (status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
  }

  params->dir->freqBins = 1 + (UINT4)(fs*T/2.0);
  params->dir->sigma = (UINT4)sigma;
  params->dir->minf = minf;
  params->dir->maxf = maxf;
  params->dir->alpha = alpha;
  params->Nsigma = Nsigma;

  LALFillCListDir(status->statusPtr, params->dir, 1E30); /* note: allocates memory for params->dir->rho from DC to Nyquist, and set to 1E30 */
  CHECKSTATUSPTR (status);

  if(delta) {
    memcpy(params->dir->d, delta, delL * sizeof(UINT4));
  
    params->dir->mdist = delta[0];
    for(i=1; i<delL; i++)
      if(delta[i] > params->dir->mdist) {params->dir->mdist = delta[i];}

    LALFree(delta);
  }
  else params->dir->mdist = 0;

  /*
   * 
   * assign the initial parameter values
   *
   */
  
  params->fs = fs;
  params->NData = (UINT4)NData;
  params->T = T;
  params->bpp = bpp;
  params->eGoal = eGoal;

  memcpy(params->name, name, strlen(name)+1);

  params->table = table;

  params->WriteBlobs = wBlobs;




  /*********** General Parameters ***************/
  fInd = 14 + delL; /* index of first general parameter */

  if(fInd + 2 > initSearchParams->argc) {
    ABORT( status, INITSEARCHH_EARGS, INITSEARCHH_MSGEARGS );
  }

  params->IFOInfo.Ndetectors = params->IFOData.Ndetectors =   params->NIFO = atoi(initSearchParams->argv[fInd]);
  if ( params->NIFO <= 0 ) {
    ABORT ( status, INITSEARCHH_ESPOS, INITSEARCHH_MSGESPOS );
  }

  if(fInd + 22 + params->NIFO != initSearchParams->argc) {
    ABORT( status, INITSEARCHH_EARGS, INITSEARCHH_MSGEARGS );
  }


  /* memory for all detectors */
  params->IFOIND = (COHIFO *)LALMalloc(params->IFOData.Ndetectors * sizeof(COHIFO));
  if(!params->IFOIND) {
    ABORT (status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
  }

  params->IFOData.data = (REAL4TimeSeries *)LALCalloc(params->IFOData.Ndetectors, sizeof(REAL4TimeSeries));
  if(!params->IFOData.data) {
    ABORT (status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
  }

  params->IFOInfo.detectors = (LALDetector *)LALCalloc(params->IFOInfo.Ndetectors, sizeof(LALDetector));
  if(!params->IFOInfo.detectors) {
    ABORT (status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
  }

  params->IFOInfo.filters = (REAL8IIRFilter **)LALCalloc(params->IFOInfo.Ndetectors, sizeof(REAL8IIRFilter *));
  if(!params->IFOInfo.filters) {
    ABORT (status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
  }

  for(i=0; i<params->NIFO; i++) {
    params->IFOData.data[i].data = NULL;
    params->IFOInfo.filters[i] = NULL;
  }

  /* read each detector name */
  for(i=0; i<params->NIFO; i++) {

    if(!strcmp(initSearchParams->argv[fInd + 1 + i], "LHO4k")) {
      params->IFOInfo.detectors[i] = lalCachedDetectors[LALDetectorIndexLHODIFF];
      params->IFOIND[i] = LHO4K;
    } else if(!strcmp(initSearchParams->argv[fInd + 1 + i], "LHO2k")) {
      params->IFOInfo.detectors[i] = lalCachedDetectors[LALDetectorIndexLHODIFF];
      params->IFOIND[i] = LHO2K;
    } else if(!strcmp(initSearchParams->argv[fInd + 1 + i], "LLO4k")) {
      params->IFOInfo.detectors[i] = lalCachedDetectors[LALDetectorIndexLLODIFF];
      params->IFOIND[i] = LLO4K;
    } else if(!strcmp(initSearchParams->argv[fInd + 1 + i], "VIRGO")) {
      params->IFOInfo.detectors[i] = lalCachedDetectors[LALDetectorIndexVIRGODIFF];
      params->IFOIND[i] = VIRGO;
    } else if(!strcmp(initSearchParams->argv[fInd + 1 + i], "GEO")) {
      params->IFOInfo.detectors[i] = lalCachedDetectors[LALDetectorIndexGEO600DIFF];
      params->IFOIND[i] = GEO;
    } else if(!strcmp(initSearchParams->argv[fInd + 1 + i], "TAMA")) {
      params->IFOInfo.detectors[i] = lalCachedDetectors[LALDetectorIndexTAMA300DIFF];
      params->IFOIND[i] = TAMA;
    } else {
      ABORT (status, INITSEARCHH_EIFO, INITSEARCHH_MSGEIFO);
    }

  }


  /*** Alpha ***/
  params->AlphaMin =  atof(initSearchParams->argv[fInd + params->NIFO + 1]);
  if ( isnan(params->AlphaMin) ) {
    ABORT ( status, INITSEARCHH_ENAN, INITSEARCHH_MSGENAN );
  }
  if ( params->AlphaMin < 0.0 || params->AlphaMin > LAL_TWOPI ) {
    ABORT ( status, INITSEARCHH_E02PI, INITSEARCHH_MSGE02PI );
  }

  params->AlphaMax =  atof(initSearchParams->argv[fInd + params->NIFO + 2]);
  if ( isnan(params->AlphaMax) ) {
    ABORT ( status, INITSEARCHH_ENAN, INITSEARCHH_MSGENAN );
  }
  if ( params->AlphaMax < 0.0 || params->AlphaMax > LAL_TWOPI ) {
    ABORT ( status, INITSEARCHH_E02PI, INITSEARCHH_MSGE02PI );
  }

  if ( params->AlphaMin > params->AlphaMax) {
    ABORT ( status, INITSEARCHH_EMMF, INITSEARCHH_MSGEMMF );
  }

  params->NAlpha =  atoi(initSearchParams->argv[fInd + params->NIFO + 3]);
  if ( params->NAlpha <= 0 ) {
    ABORT ( status, INITSEARCHH_ESPOS, INITSEARCHH_MSGESPOS );
  }


  /*** Delta ***/
  params->DeltaMin =  atof(initSearchParams->argv[fInd + params->NIFO + 4]);
  if ( isnan(params->DeltaMin) ) {
    ABORT ( status, INITSEARCHH_ENAN, INITSEARCHH_MSGENAN );
  }
  if ( fabs(params->DeltaMin) > LAL_PI) {
    ABORT ( status, INITSEARCHH_E02PI, INITSEARCHH_MSGE02PI );
  }

  params->DeltaMax =  atof(initSearchParams->argv[fInd + params->NIFO + 5]);
  if ( isnan(params->DeltaMax) ) {
    ABORT ( status, INITSEARCHH_ENAN, INITSEARCHH_MSGENAN );
  }
  if ( fabs(params->DeltaMax) > LAL_PI) {
    ABORT ( status, INITSEARCHH_E02PI, INITSEARCHH_MSGE02PI );
  }

  if ( params->DeltaMin > params->DeltaMax) {
    ABORT ( status, INITSEARCHH_EMMF, INITSEARCHH_MSGEMMF );
  }

  params->NDelta =  atoi(initSearchParams->argv[fInd + params->NIFO + 6]);
  if ( params->NDelta <= 0 ) {
    ABORT ( status, INITSEARCHH_ESPOS, INITSEARCHH_MSGESPOS );
  }



  /*** Psi ***/
  params->PsiMin =  atof(initSearchParams->argv[fInd + params->NIFO + 7]);
  if ( isnan(params->PsiMin) ) {
    ABORT ( status, INITSEARCHH_ENAN, INITSEARCHH_MSGENAN );
  }
  if ( fabs(params->PsiMin) > LAL_TWOPI ) {
    ABORT ( status, INITSEARCHH_E02PI, INITSEARCHH_MSGE02PI );
  }

  params->PsiMax =  atof(initSearchParams->argv[fInd + params->NIFO + 8]);
  if ( isnan(params->PsiMax) ) {
    ABORT ( status, INITSEARCHH_ENAN, INITSEARCHH_MSGENAN );
  }
  if ( fabs(params->PsiMax) > LAL_TWOPI ) {
    ABORT ( status, INITSEARCHH_E02PI, INITSEARCHH_MSGE02PI );
  }

  if ( params->PsiMin > params->PsiMax) {
    ABORT ( status, INITSEARCHH_EMMF, INITSEARCHH_MSGEMMF );
  }

  params->NPsi =  atoi(initSearchParams->argv[fInd + params->NIFO + 9]);
  if ( params->NPsi <= 0 ) {
    ABORT ( status, INITSEARCHH_ESPOS, INITSEARCHH_MSGESPOS );
  }


  /*** plusDotcross ***/
  params->plusDotcrossMin =  atof(initSearchParams->argv[fInd + params->NIFO + 10]);
  if ( isnan(params->plusDotcrossMin) ) {
    ABORT ( status, INITSEARCHH_ENAN, INITSEARCHH_MSGENAN );
  }
  if ( fabs(params->plusDotcrossMin) > LAL_PI) {
    ABORT ( status, INITSEARCHH_E02PI, INITSEARCHH_MSGE02PI );
  }

  params->plusDotcrossMax =  atof(initSearchParams->argv[fInd + params->NIFO + 11]);
  if ( isnan(params->plusDotcrossMax) ) {
    ABORT ( status, INITSEARCHH_ENAN, INITSEARCHH_MSGENAN );
  }
  if ( fabs(params->plusDotcrossMax) > LAL_PI) {
    ABORT ( status, INITSEARCHH_E02PI, INITSEARCHH_MSGE02PI );
  }

  if ( params->plusDotcrossMin > params->plusDotcrossMax) {
    ABORT ( status, INITSEARCHH_EMMF, INITSEARCHH_MSGEMMF );
  }

  params->NplusDotcross =  atoi(initSearchParams->argv[fInd + params->NIFO + 12]);
  if ( params->NplusDotcross <= 0 ) {
    ABORT ( status, INITSEARCHH_ESPOS, INITSEARCHH_MSGESPOS );
  }



  /*** plus2cross ***/
  params->plus2crossMin =  atof(initSearchParams->argv[fInd + params->NIFO + 13]);
  if ( isnan(params->plus2crossMin) ) {
    ABORT ( status, INITSEARCHH_ENAN, INITSEARCHH_MSGENAN );
  }

  params->plus2crossMax =  atof(initSearchParams->argv[fInd + params->NIFO + 14]);
  if ( isnan(params->plus2crossMax) ) {
    ABORT ( status, INITSEARCHH_ENAN, INITSEARCHH_MSGENAN );
  }

  if ( params->plus2crossMin > params->plus2crossMax) {
    ABORT ( status, INITSEARCHH_EMMF, INITSEARCHH_MSGEMMF );
  }

  params->Nplus2cross =  atoi(initSearchParams->argv[fInd + params->NIFO + 15]);


  /* vectors of sampling points */
  params->Alpha = (REAL8 *)LALMalloc(params->NAlpha * sizeof(REAL8));
  if(!(params->Alpha)) {
    ABORT (status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
  }
  if(params->NAlpha > 1) {
    for(i=0;i<params->NAlpha;i++) {
      params->Alpha[i] = params->AlphaMin + (params->AlphaMax - params->AlphaMin)*(REAL8)i / (REAL8)(params->NAlpha - 1);
    }
  } else {
    params->Alpha[0] = params->AlphaMin;
  }

  params->Psi = (REAL8 *)LALMalloc(params->NPsi * sizeof(REAL8));
  if(!(params->Psi)) {
    ABORT (status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
  }
  if(params->NPsi > 1) {
    for(i=0;i<params->NPsi;i++) {
      params->Psi[i] = params->PsiMin + (params->PsiMax - params->PsiMin)*(REAL8)i / (REAL8)(params->NPsi - 1);
    }
  } else {
    params->Psi[0] = params->PsiMin;
  }

  params->Delta = (REAL8 *)LALMalloc(params->NDelta * sizeof(REAL8));
  if(!(params->Delta)) {
    ABORT (status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
  }
  if(params->NDelta > 1) {
    for(i=0;i<params->NDelta;i++) {
      params->Delta[i] = asin ( sin(params->DeltaMin) + (sin(params->DeltaMax) - sin(params->DeltaMin))*(REAL8)i / (REAL8)(params->NDelta - 1 ) ); 
    }
  } else {
    params->Delta[0] = params->DeltaMin;
  }

  params->plusDotcross = (REAL8 *)LALMalloc(params->NplusDotcross * sizeof(REAL8));
  if(!(params->plusDotcross)) {
    ABORT (status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
  }
  if(params->NplusDotcross > 1) {
    for(i=0;i<params->NplusDotcross;i++) {
      params->plusDotcross[i] = params->plusDotcrossMin + (params->plusDotcrossMax - params->plusDotcrossMin)*(REAL8)i / (REAL8)(params->NplusDotcross - 1 ); 
    }
  } else {
    params->plusDotcross[0] = params->plusDotcrossMin;
  }

  params->plus2cross = (REAL8 *)LALMalloc(params->Nplus2cross * sizeof(REAL8));
  if(!(params->plus2cross)) {
    ABORT (status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
  }
  if(params->Nplus2cross > 1) {
    for(i=0;i<params->Nplus2cross;i++) {
      params->plus2cross[i] = params->plus2crossMin * pow(params->plus2crossMax/params->plus2crossMin, (REAL8)i / (REAL8)(params->Nplus2cross-1));
    }
  } else {
    params->plus2cross[0] = params->plus2crossMin;
  }

  /* tf-plane boundaries */
  params->ccf = ccf = atof(initSearchParams->argv[fInd + params->NIFO + 16]);
  if ( ccf < 0.0 ) {
    ABORT ( status, INITSEARCHH_ESPOS, INITSEARCHH_MSGESPOS );
  }

  params->cbw = cbw = atof(initSearchParams->argv[fInd + params->NIFO + 17]);
  if ( cbw < 0.0 ) {
    ABORT ( status, INITSEARCHH_ESPOS, INITSEARCHH_MSGESPOS );
  }

  cmint = atof(initSearchParams->argv[fInd + params->NIFO + 18]);
  if ( cmint < 0.0 ) {
    ABORT ( status, INITSEARCHH_ESPOS, INITSEARCHH_MSGESPOS );
  }

  cmaxt = atof(initSearchParams->argv[fInd + params->NIFO + 19]);
  if ( cmaxt < 0.0 ) {
    ABORT ( status, INITSEARCHH_ESPOS, INITSEARCHH_MSGESPOS );
  }

  if( cmint > cmaxt ) {
    ABORT ( status, INITSEARCHH_EMMF, INITSEARCHH_MSGEMMF );
  }

  params->cminf = (UINT4)floor(2.0 * ccf * params->T) - 1; /* minf + maxf */
  tmpU4 = (UINT4)floor(cbw * params->T) - 1; /* maxf - minf */

  params->cmaxf = (tmpU4 + params->cminf)/2;
  params->cminf = (params->cminf - tmpU4)/2;

  params->cmintR8 = cmint; /* 'coz don't know start time */
  params->cmaxtR8 = cmaxt;

  params->IFOInfo.nPreProcessed = (UINT2)atoi(initSearchParams->argv[fInd + params->NIFO + 20]);

  params->SearchMethod = (UINT4)atoi(initSearchParams->argv[fInd + params->NIFO + 21]);

  params->searchMaster = initSearchParams->nodeClass;

  params->numSlaves = 0;

  params->IFOInfo.preProcessed = 0; /* preProcessing will be done only once on each slave */

  params->IFOInfo.CMat = (REAL8 **)LALMalloc(params->NIFO * sizeof(REAL8 *));
  if(!(params->IFOInfo.CMat)) {
    ABORT (status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
  }

  for(i=0;i<params->NIFO;i++) {
    params->IFOInfo.CMat[i] = (REAL8 *)LALMalloc(params->NIFO * sizeof(REAL8));
    if(!(params->IFOInfo.CMat[i])) {
      ABORT (status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
    }

    bzero(params->IFOInfo.CMat[i], params->NIFO * sizeof(REAL8));

  }

  /*
  printf("%g\n",cosh(acosh(fabs(-3.02993))/3.0));
  */

  DETATCHSTATUSPTR (status);
  RETURN (status);
}

