/*----------------------------------------------------------------------- 
 * 
 * File Name: ApplySearch.c
 *
 * Author: Brown, D. A.
 * 
 * Revision: $Id: ApplySearch.c,v 1.42 2003/06/20 22:21:00 jolien Exp $
 * 
 *-----------------------------------------------------------------------
 */

#include "ApplySearch.h"

NRCSID( APPLYSEARCHC, PKG "$Id: ApplySearch.c,v 1.42 2003/06/20 22:21:00 jolien Exp $" );


void
LALApplySearch(
    LALStatus             *status,
    LALSearchOutput       *output,
    LALSearchInput        *input,
    LALApplySearchParams  *applyParams
              )
{
  UINT4                         i;
  InspiralSearchParams         *params          = NULL;
  InspiralMasterParams        *masterParamsPtr = NULL;
  InspiralSlaveParams         *slaveParamsPtr  = NULL;
  SnglInspiralTable            *eventList       = NULL;


  INITSTATUS( status, "LALApplySearch", APPLYSEARCHC );
  ATTATCHSTATUSPTR( status );


 /*
  *
  * check input, output and parameters
  *
  */


  ASSERT( output, status, 
      APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );
  ASSERT( !(output->result), status, 
      APPLYSEARCHH_ENNUL, APPLYSEARCHH_MSGENNUL );

  ASSERT( applyParams, status, 
      APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );
  ASSERT( applyParams->mpiParams, status, 
      APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );
  ASSERT( applyParams->searchParams, status, 
      APPLYSEARCHH_ENULL, APPLYSEARCHH_MSGENULL );

  /* pointers to parameter structures */
  params = (InspiralSearchParams *) applyParams->searchParams;
  masterParamsPtr = &(params->masterParams);
  slaveParamsPtr  = &(params->slaveParams);


 /*
  *
  * set up or update the mpi communications within the search comm
  *
  */


  /* if search master, get the initial number of nodes in the serach comm */
  if ( params->searchMaster )
  {
    /* only do this the first time the master is called */
    if ( ! masterParamsPtr->numSlaves )
    {
      /* allocate memory and check for success */
      masterParamsPtr->numSlaves = (UINT4 *) LALCalloc( 1, sizeof(UINT4) );
      if( ! masterParamsPtr->numSlaves )
      {
        ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
      }

      /* get the number of search slaves */
      {
        int itmp;
        MPI_Comm_size( *(applyParams->mpiParams->comm), &itmp );
        *masterParamsPtr->numSlaves = itmp;
      }

      /* subrtact the master to get the number of slaves */
      params->info.numNodes = *(masterParamsPtr->numSlaves) -= 1;

      if ( params->inspiralDebugFlag == fcBankMinimalMatch )
      {
        /* create the bank sim vector if it is required (master is 0) */
        LALU4CreateVector( status->statusPtr, &(masterParamsPtr->bankSentVec),
            *(masterParamsPtr->numSlaves) + 1 );
        CHECKSTATUSPTR( status );

        memset( masterParamsPtr->bankSentVec->data, 0, 
            masterParamsPtr->bankSentVec->length  * sizeof(UINT4) );

        /* set the number of templates to filter: faked for progress */
        masterParamsPtr->numTmpltsToFilter = 
          *(masterParamsPtr->numSlaves) * (params->simCount + 1);
      }
    }
  }


 /*
  *
  * set up the internal parameters structures
  *
  */


  if ( params->searchMaster )
  {
    masterParamsPtr->mpiComm = applyParams->mpiParams->comm;
  }
  else /* search slave */
  {
    slaveParamsPtr->mpiComm = applyParams->mpiParams->comm;
  }


  /*
   *
   * execute the appropriate master or slave engine
   *
   */


  /* set the recieved event list and output to NULL */
  eventList             = NULL;
  output->numOutput     = 0;
  output->result        = NULL;

  if ( params->searchMaster )
  {
    UINT4 numEvents = 0;

    /* call the inspiral master engine: this manages the distribution   */
    /* of templates and progess information. input and output are       */
    /* handled by the slaves unless we are doing a bank simulation      */
    InspiralMaster( status->statusPtr, &eventList, &numEvents, 
        masterParamsPtr );
    CHECKSTATUSPTR( status );

    params->info.numEvents += numEvents;

    /* compute progress information for wrapperAPI */
    if ( *(masterParamsPtr->numSlaves) )
    {
      output->fracRemaining = 
        ( (REAL4) masterParamsPtr->numTmpltsToFilter - 
          (REAL4) masterParamsPtr->numTmpltsFiltered ) / 
        (REAL4) masterParamsPtr->numTmpltsToFilter;
      output->notFinished   = 1;
    }
    else
    {
      output->fracRemaining = 0;
      output->notFinished   = 0;
    }
  }
  else /* search slave */
  {
    /* set the pointer to the wrapperAPIs not finished flag     */
    slaveParamsPtr->notFinished = &(output->notFinished);

    if ( params->dumpData )
    {
      /* write out the input data before we have done anything to it */
      REAL4TimeSeries chan;
      COMPLEX8FrequencySeries resp;

      memcpy( &chan, params->dataSegVec->data->chan, 
          sizeof(REAL4TimeSeries) );
      memcpy( &resp, params->dataSegVec->data->resp, 
          sizeof(COMPLEX8FrequencySeries) );

      snprintf( chan.name, LALNameLength * sizeof(CHAR),
          "%s:channel:input", params->dataSegVec->data->chan->name );
      snprintf( resp.name, LALNameLength * sizeof(CHAR),
          "%s:response:input", params->dataSegVec->data->resp->name );
      
      BuildREAL4TimeSeriesOutPut( status->statusPtr, output, &chan);
      CHECKSTATUSPTR( status );
      
      BuildCOMPLEX8FrequencySeriesOutPut( status->statusPtr, output, &resp);
      CHECKSTATUSPTR( status );
      
      if ( slaveParamsPtr->haveSpec )
      {
        REAL4FrequencySeries spec;
        memcpy( &spec, params->dataSegVec->data->spec, 
            sizeof(REAL4FrequencySeries) );
        snprintf( spec.name, LALNameLength * sizeof(CHAR),
            "%s:spectrum:input", params->dataSegVec->data->spec->name );
        BuildREAL4FrequencySeriesOutPut( status->statusPtr, output, &spec);
        CHECKSTATUSPTR( status );
      }
    }

    /* call the inspiral slave engine: this will return with    */
    /* output->notFinished set appropriately and possible a     */
    /* a list of inspiral events                                */
    InspiralSlave( status->statusPtr, &eventList, 
        params->dataSegVec, slaveParamsPtr );
    CHECKSTATUSPTR( status );

    /* if ConditionData() has created any injection objects destroy */
    /* them now to prevent them from being injected multiple times  */
    /* into the input data stream and screwing up the injections    */
    if ( slaveParamsPtr->simParams ) 
    {
      while ( slaveParamsPtr->simParams->injectEvent )
      {
        InspiralInjection *thisEvent = slaveParamsPtr->simParams->injectEvent;
        slaveParamsPtr->simParams->injectEvent =
          slaveParamsPtr->simParams->injectEvent->next;
        LALFree( thisEvent );
      }
    }

    if ( params->dumpData )
    {
      /* write out the input data after we have done injections into it */
      REAL4TimeSeries chan;
      COMPLEX8FrequencySeries resp;

      memcpy( &chan, params->dataSegVec->data->chan, 
          sizeof(REAL4TimeSeries) );
      memcpy( &resp, params->dataSegVec->data->resp, 
          sizeof(COMPLEX8FrequencySeries) );

      snprintf( chan.name, LALNameLength * sizeof(CHAR),
          "%s:channel:output", params->dataSegVec->data->chan->name );
      snprintf( resp.name, LALNameLength * sizeof(CHAR),
          "%s:response:output", params->dataSegVec->data->resp->name );
      
      BuildREAL4TimeSeriesOutPut( status->statusPtr, output, &chan);
      CHECKSTATUSPTR( status );

      BuildCOMPLEX8FrequencySeriesOutPut( status->statusPtr, output, &resp);
      CHECKSTATUSPTR( status );

      if ( slaveParamsPtr->haveSpec )
      {
        REAL4FrequencySeries spec;
        memcpy( &spec, params->dataSegVec->data->spec, 
            sizeof(REAL4FrequencySeries) );
        snprintf( spec.name, LALNameLength * sizeof(CHAR),
            "%s:spectrum:output", params->dataSegVec->data->spec->name );
        BuildREAL4FrequencySeriesOutPut( status->statusPtr, output, &spec);
        CHECKSTATUSPTR( status );
      }
    }

    /* make sure we only dump the input and processed data once */
    params->dumpData = 0;

    /* since we are operating in searchMaster/searchSlave mode  */
    /* fracRemaining is ignored on the search slaves. just set  */
    /* it to a sensible value                                   */
    output->fracRemaining = 0;

  } /* end if master/slave */


 /*
  *
  * if events are returned, generate the outPut structure 
  *
  */


  if ( eventList )
  {
    BuildInspiralOutPut( status->statusPtr, output, eventList );
    CHECKSTATUSPTR( status );

    while ( eventList )
    {
      SnglInspiralTable *thisEvent = eventList;
      eventList = eventList->next;
      LALFree (thisEvent);
      thisEvent = NULL;
    }
  }

  /* auxillary outputs (i.e. non inspiral events ) */
  if ( ! params->searchMaster )
  {
    /* if the filter output was requested and this is not the last loop */
    /* through the slave, append the snrsq vec to the output strcuture  */
    if ( params->slaveParams.filterParams->rhosqVec && output->notFinished )
    {
      BuildREAL4TimeSeriesOutPut( status->statusPtr, output, 
          params->slaveParams.filterParams->rhosqVec );
      CHECKSTATUSPTR( status );

      if ( params->slaveParams.filterParams->chisqVec )
      {
        REAL4TimeSeries chisqOutput;
        memset( &chisqOutput, 0, sizeof(REAL4TimeSeries) );
        chisqOutput.data = params->slaveParams.filterParams->chisqVec;
        chisqOutput.deltaT = params->dataSegVec->data->chan->deltaT;
        memcpy( &chisqOutput.epoch, 
            &(params->slaveParams.filterParams->rhosqVec->epoch), 
            sizeof(LIGOTimeGPS) );
        snprintf( chisqOutput.name, LALNameLength * sizeof(CHAR),
            "%s:chisq:output", params->dataSegVec->data->chan->name );

        BuildREAL4TimeSeriesOutPut( status->statusPtr, output, &chisqOutput );
        CHECKSTATUSPTR( status );
      }
    }

    /* create the standard candle output if it exists, */
    /* but make sure we only do this once              */
    if ( params->slaveParams.candlePtr )
    {
      if ( params->slaveParams.filterParams->rhosqVec )
      {
        BuildCandleOutPut( status->statusPtr, output, 
            params->slaveParams.candlePtr, &(params->info) );
        CHECKSTATUSPTR( status );
      }

      /* if we are filtering the gravity wave channel, store */
      /* the standard candle in the summ_value table         */
      if ( strstr( params->chan.name, "LSC-AS_Q" ) )
      {
        BuildCandleDataBaseOutPut( status->statusPtr, output,
            params->slaveParams.candlePtr, &(params->info) );
        CHECKSTATUSPTR( status );
      }

      LALFree( params->slaveParams.candlePtr );
      params->slaveParams.candlePtr = NULL;
    }

    if ( ! output->notFinished )
    {
      UINT4     commRank;
      int itmp;
      
      /* get the slaves rank */
      MPI_Comm_rank( *(applyParams->mpiParams->comm), &itmp );
      commRank = itmp;

#ifdef INSPIRAL_SO_DEBUG
      fprintf( stdout, "slave %d finished\n", commRank );
      fflush( stdout );
#endif

      if ( commRank == 1 )
      {
        enum { numsumvars = 4 };
        dataBase               *dbase;
        UINT4                   thisOutput = output->numOutput;
        SearchSummaryVariables  summvars[numsumvars];
        REAL8                   summvars_value[numsumvars];

        /* create the search summ value table on the first slave only */
        output->numOutput += numsumvars;
        output->result = (outPut *) 
          LALRealloc( output->result, output->numOutput * sizeof( outPut ) );
        if ( ! output->result )
        {
          ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
        }

        memset( output->result + thisOutput, 0, numsumvars * sizeof( outPut ) );

        summvars[0].name  = "inspiral_response_type";
        summvars[1].name  = "inspiral_response_time";
        summvars[2].name  = "inspiral_response_time_ns";
        summvars[3].name  = "inspiral_spec_type";
        summvars_value[0] =  (REAL8) params->responseType;
        summvars_value[1] =  (REAL8) params->responseEpoch.gpsSeconds;
        summvars_value[2] =  (REAL8) params->responseEpoch.gpsNanoSeconds;
        summvars_value[3] =  (REAL8) params->specType;

        for ( i = 0; i < numsumvars; ++i )
        {
          output->result[thisOutput + i ].search = binaryInspiral;
          output->result[thisOutput + i ].significant = 1;
          summvars[i].value = summvars_value + i;
          summvars[i].string = NULL;  
          dbase = NULL;

          LALCreateSearchSummvarsDataBase( status->statusPtr, 
              &dbase, summvars + i );
          CHECKSTATUSPTR( status );
          output->result[thisOutput + i].results = dbase;
        }
      }
    }
  }
  else if ( ! output->notFinished )
  {
    /* if this is a real search then create an entry for the */
    /* search summary table now that the job has finished    */
    dataBase   *dbase = NULL;
    LALBLOB     blob;
    UINT4       thisOutput = output->numOutput++;

    output->result = (outPut *) 
      LALRealloc( output->result, output->numOutput * sizeof( outPut ) );
    if ( ! output->result )
    {
      ABORT( status, APPLYSEARCHH_EALOC, APPLYSEARCHH_MSGEALOC );
    }

    memset( output->result + thisOutput, 0, sizeof( outPut ) );
    
    output->result[thisOutput].search = binaryInspiral;
    output->result[thisOutput].significant = 1;

    params->info.dsoName = "ldasinspiral";

    /* column "comment" */
    LALCreateNextDBEntry( status->statusPtr, &dbase, 
        "search_summary", "comment" );
    CHECKSTATUSPTR( status );
    blob.data = params->chan.name;
    blob.size = strlen( params->chan.name );
    LALCreateDBEntryBLOB( status->statusPtr, dbase, &blob, 1 );
    CHECKSTATUSPTR( status );
    output->result[thisOutput].results = dbase;

    LALCreateSearchSummaryDataBase( status->statusPtr, 
        &(dbase->next), &(params->info) );
    CHECKSTATUSPTR( status );
  }


  /* normal exit */
  DETATCHSTATUSPTR (status);
  RETURN (status);
}
