#include <math.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <LALWrapperInterface.h>
#include <ExtractSeries.h>
#include <BuildDB.h>
#include <lal/LALStdlib.h>
#include <lal/AVFactories.h>
#include <lal/VectorOps.h>
#include <lal/TimeFreqFFT.h>
#include <lal/Units.h>

NRCSID( TESTC, "$Id: Test.c,v 1.4 2003/06/20 22:00:08 jolien Exp $" );

typedef struct
tagTestParameters
{
  int nodeClass;
  COMPLEX8FrequencySeries *strain;
}
TestParameters;

void
LALInitSearch(
    LALStatus            *status,
    void                **searchParams,
    LALInitSearchParams  *initSearchParams
    )
{
  TestParameters *testParams;
  char args[1024] = "";
  int rank;
  int arg;
  INITSTATUS( status, "LALInitSearch", TESTC );

  MPI_Comm_rank( MPI_COMM_WORLD, &rank );

  fprintf( stderr, "Rank %d: Entering LALInitSearch\n", rank );
  fprintf( stderr, "Rank %d: Sleeping for 10 seconds\n", rank );
  sleep( 10 );
  fprintf( stderr, "Rank %d: Woke from sleep\n", rank );

  /* NOTE: argc is defined funny in wrapperAPI */
  for ( arg = 0; arg < initSearchParams->argc; ++arg )
  {
    strncat( args, initSearchParams->argv[arg],
        sizeof( args ) - strlen( args ) - 1 );
    strncat( args, " ", sizeof( args ) - strlen( args ) - 1 );
  }
  fprintf( stderr, "Rank %d: Received args '%s'\n", rank, args );

  *searchParams = testParams = LALCalloc( 1, sizeof( *testParams ) );
  testParams->nodeClass = initSearchParams->nodeClass;

  fprintf( stderr, "Rank %d: Leaving LALInitSearch\n", rank );

  RETURN( status );
}

void
LALFinalizeSearch(
    LALStatus  *status,
    void      **searchParams
    )
{
  TestParameters *testParams;
  int rank;
  INITSTATUS( status, "LALFinalizeSearch", TESTC );
  ATTATCHSTATUSPTR( status );

  MPI_Comm_rank( MPI_COMM_WORLD, &rank );

  fprintf( stderr, "Rank %d: Entering LALFinalizeSearch\n", rank );
  testParams = *searchParams;
  if ( testParams )
  {
    if ( testParams->strain )
    {
      if ( testParams->strain->data )
      {
        LALCDestroyVector( status->statusPtr, &testParams->strain->data );
        CHECKSTATUSPTR( status );
      }
      LALFree( testParams->strain );
      testParams->strain = NULL;
    }
    LALFree( testParams );
    *searchParams = NULL;
  }
  fprintf( stderr, "Rank %d: Leaving LALFinalizeSearch\n", rank );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}


void
LALConditionData(
    LALStatus      *status,
    LALSearchInput *inout,
    void           *searchParams,
    LALMPIParams   *mpiParams
    )
{
  const LALUnit strainPerCount = {0,{0,0,0,0,0,1,-1},{0,0,0,0,0,0,0}};
  LALUnitPair upair;
  TestParameters  *testParams;
  multiDimData    *adcmdd = NULL;
  REAL4Vector      adcvec;
  REAL4TimeSeries  adcser;
  COMPLEX8FrequencySeries response;
  RealFFTPlan *plan;
  REAL4 dt;
  UINT4 npt;
  int rank;

  INITSTATUS( status, "LALConditionData", TESTC );
  ATTATCHSTATUSPTR( status );

  MPI_Comm_rank( MPI_COMM_WORLD, &rank );

  fprintf( stderr, "Rank %d: Entering LALConditionData\n", rank );

  testParams  = searchParams;
  adcvec.data = NULL;
  adcser.data = &adcvec;
  adcser.sampleUnits = lalADCCountUnit;
  LALFindSequenceByName( status->statusPtr, &adcmdd, inout, "channel" );
  CHECKSTATUSPTR( status );
  LALExtractREAL4TimeSeries( status->statusPtr, &adcser, adcmdd );
  CHECKSTATUSPTR( status );

  npt = adcser.data->length;
  dt  = npt * adcser.deltaT;

  testParams->strain = LALCalloc( 1, sizeof( *testParams->strain ) );
  if ( ! testParams->strain )
  {
    ABORT( status, 1, "Memory allocation error" );
  }
  LALCCreateVector( status->statusPtr, &testParams->strain->data, npt / 2 + 1 );
  CHECKSTATUSPTR( status );

  LALCreateForwardRealFFTPlan( status->statusPtr, &plan, npt, 0 );
  CHECKSTATUSPTR( status );
  LALTimeFreqRealFFT( status->statusPtr, testParams->strain, &adcser, plan );
  CHECKSTATUSPTR( status );
  LALDestroyRealFFTPlan( status->statusPtr, &plan );
  CHECKSTATUSPTR( status );

  response.epoch       = adcser.epoch;
  response.f0          = adcser.f0;
  response.deltaF      = 1.0 / dt;
  response.sampleUnits = strainPerCount;
  response.data        = NULL;
  strncpy( response.name, "response", sizeof( response.name ) );
  LALCCreateVector( status->statusPtr, &response.data, npt / 2 + 1 );
  CHECKSTATUSPTR( status );
  LALExtractResponse( status->statusPtr, &response, inout, NULL );
  CHECKSTATUSPTR( status );
  LALCCVectorMultiply( status->statusPtr, testParams->strain->data,
      testParams->strain->data, response.data );
  CHECKSTATUSPTR( status );
  upair.unitOne = &testParams->strain->sampleUnits;
  upair.unitTwo = &response.sampleUnits;
  LALUnitMultiply( status->statusPtr, &testParams->strain->sampleUnits, &upair );
  CHECKSTATUSPTR( status );
  LALCDestroyVector( status->statusPtr, &response.data );
  CHECKSTATUSPTR( status );

  fprintf( stderr, "Rank %d: Leaving LALConditionData\n", rank );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}


void
LALApplySearch(
    LALStatus             *status,
    LALSearchOutput       *output,
    LALSearchInput        *input,
    LALApplySearchParams  *applyParams
    )
{
  TestParameters *testParams;
  int rank;

  INITSTATUS( status, "LALApplySearch", TESTC );
  ATTATCHSTATUSPTR( status );

  MPI_Comm_rank( MPI_COMM_WORLD, &rank );

  fprintf( stderr, "Rank %d: Entering LALApplySearch\n", rank );

  testParams = applyParams->searchParams;

  output->notFinished   = 0;
  output->fracRemaining = 0;
  output->numOutput     = 0;
  output->result        = NULL;

  if ( testParams->nodeClass ) /* only the search master does anything */
  {
    INT8 tend;
    SearchSummaryInfo info;

    fprintf( stderr, "Rank %d: Building output\n", rank );

    output->numOutput = 2;
    output->result = LALCalloc( output->numOutput, sizeof( *output->result ) );
    if ( ! output->result )
    {
      ABORT( status, 1, "Memory allocation error" );
    }

    /* first output: search summary */
    output->result[0].search = burst;
    output->result[0].significant = 1;
    tend  = (INT8)( 1000000000 ) * (INT8)( testParams->strain->epoch.gpsSeconds );
    tend += (INT8)( testParams->strain->epoch.gpsNanoSeconds );
    tend += (INT8)( 1e9 / testParams->strain->deltaF );
    info.dsoName                     = "ldastest";
    info.inputStartTime              = testParams->strain->epoch;
    info.inputEndTime.gpsSeconds     = (INT4)( tend / (INT8)(1000000000) );
    info.inputEndTime.gpsNanoSeconds = (INT4)( tend % (INT8)(1000000000) );
    info.outputStartTime             = info.inputStartTime;
    info.outputEndTime               = info.inputEndTime;
    info.numEvents                   = 1;
    info.numNodes                    = 1; /* only one node actually used */
    LALCreateSearchSummaryDataBase( status->statusPtr,
        &output->result[0].results, &info );
    CHECKSTATUSPTR( status );

    /* second output: results */
    output->result[1].search = burst;
    output->result[1].significant = 1;
    output->result[1].optional = LALCalloc( 1, sizeof( *(output->result[1].optional) ) );
    output->result[1].optional[0].space = freqD;
    output->result[1].optional[0].type = complex_8;
    output->result[1].optional[0].range.dFreq.numberSamples =
      testParams->strain->data->length;
    output->result[1].optional[0].range.dFreq.gpsStartTimeSec =
      info.outputStartTime.gpsSeconds;
    output->result[1].optional[0].range.dFreq.gpsStartTimeNan =
      info.outputStartTime.gpsNanoSeconds;
    output->result[1].optional[0].range.dFreq.gpsStopTimeSec =
      info.outputEndTime.gpsSeconds;
    output->result[1].optional[0].range.dFreq.gpsStopTimeNan =
      info.outputEndTime.gpsNanoSeconds;
    output->result[1].optional[0].range.dFreq.startFreq = 0;
    output->result[1].optional[0].range.dFreq.stopFreq = 
      ( testParams->strain->data->length - 1 ) * testParams->strain->deltaF;
    output->result[1].optional[0].range.dFreq.freqStepSize = 
      testParams->strain->deltaF;
    output->result[1].optional[0].range.dFreq.baseFreq = 0;
    output->result[1].optional[0].numberDimensions = 1;
    output->result[1].optional[0].dimensions =
      LALCalloc( output->result[1].optional[0].numberDimensions,
          sizeof( *(output->result[1].optional[0].data.complex8) ) );
    output->result[1].optional[0].dimensions[0] =
      testParams->strain->data->length;
    output->result[1].optional[1] = output->result[1].optional[1];
    output->result[1].optional[0].data.complex8 =
      LALCalloc( testParams->strain->data->length,
          sizeof( *(output->result[1].optional[0].data.complex8) ) );
    strcpy( output->result[1].optional[0].name, testParams->strain->name );
    {
      CHAR tmpStr[64] = "";
      CHARVector tmpVec;
      tmpVec.length = 64;
      tmpVec.data = tmpStr;
      LALUnitAsString( status->statusPtr, &tmpVec,
          &testParams->strain->sampleUnits );
      CHECKSTATUSPTR( status );
      strcpy( output->result[1].optional[0].units, tmpStr );
    }
    {
      UINT4 i;
      for ( i = 0; i < testParams->strain->data->length; ++i )
      {
        output->result[1].optional[0].data.complex8[i].re = 
          testParams->strain->data->data[i].re;
        output->result[1].optional[0].data.complex8[i].im = 
          testParams->strain->data->data[i].im;
      }
    }
  }

  fprintf( stderr, "Rank %d: Leaving LALApplySearch\n", rank );

  DETATCHSTATUSPTR( status );
  RETURN( status );
}
