#if 0  /* autodoc block */

<lalVerbatim file="happyAPICV">
$Id: happyAPI.c,v 1.7 2003/07/03 18:30:56 jolien Exp $
</lalVerbatim>

<lalLaTeX>

\subsection{Program \texttt{happyAPI.c}}
\label{ss:happyAPI.c}

A program that does a crude approximation to what \verb+wrapperAPI+ does:
dynamically loads a shared-object library and executes the functions required
by \verb+wrapperAPI+.  This program is used to test a shared object that is to
be run by \verb+wrapperAPI+.

\subsubsection*{Usage}
\begin{verbatim}
happyAPI sofile [filterparams]
\end{verbatim}
where \verb+sofile+ is the shared object library
(\verb+lib+$\langle$whatever$\rangle$\verb+.so+) to test and filterparams are
the optional filter parameters.

\subsubsection*{Description}
\subsubsection*{Exit codes}
\begin{tabular}{|c|l|}
\hline
 Code & Explanation                   \\
\hline
\ttfamily 0 & Success, normal exit.         \\
\ttfamily 1 & An error was returned by a shared object function. \\
\ttfamily 2 & An internal error. \\
\hline
\end{tabular}

\vfill{\footnotesize\input{happyAPICV}}

</lalLaTeX>

#endif /* autodoc block */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <dlfcn.h>
#include <mpi.h>
#include <wrapperInterface.h>
#include <LALWrapperConfig.h>

#define CHECK_CODE( func, code, mesg )                                         \
  do switch ( code ) {                                                         \
    case -1: fprintf( stderr, "%s: Warning: %s\n", func, mesg ) ; break ;      \
    case  0: fprintf( stderr, "%s: Nominal.\n", func ) ; break ;               \
    case  1: fprintf( stderr, "%s: Error: %s\n", func, mesg ) ; return 1 ;     \
    default: fprintf( stderr, "%s: Invalid code %d\n", func, code ); return 2; \
  } while ( 0 )

typedef INT4 (init_f)( CHAR **, InitParams * );
typedef INT4 (cond_f)( CHAR **, inPut *, SearchParams * );
typedef INT4 (appl_f)( CHAR **, inPut *, SearchOutput *, SearchParams * );
typedef INT4 (free_f)( CHAR **, SearchOutput * );
typedef INT4 (fini_f)( CHAR ** );

int happy( int argc, char **argv, MPI_Comm comm );

int main( int argc, char **argv )
{
  int rank;
  int size;
  int code;

  MPI_Init( &argc, &argv );

  MPI_Comm_rank( MPI_COMM_WORLD, &rank );
  MPI_Comm_size( MPI_COMM_WORLD, &size );

  if ( rank )
  {
    char fname[64];
    snprintf( fname, sizeof( fname ), "Comrad%03d.err", rank );
    freopen( fname, "w", stderr );
    snprintf( fname, sizeof( fname ), "Comrad%03d.out", rank );
    freopen( fname, "w", stdout );
  }

  code = happy( argc, argv, MPI_COMM_WORLD );

  /* communicate comrad codes to commissar */
  if ( rank )
  {
    MPI_Send( &code, 1, MPI_INT, 0, 1, MPI_COMM_WORLD );
  }
  else
  {
    int comrad;
    fprintf( stderr, "Commissar code: %d\n", code );
    for ( comrad = 1; comrad < size; ++comrad )
    {
      int        comradCode;
      MPI_Status status;
      MPI_Recv( &comradCode, 1, MPI_INT, comrad, 1, MPI_COMM_WORLD, &status );
      fprintf( stderr, "Comrad%03d code: %d\n", comrad, comradCode );
      code |= comradCode;
    }
  }

  fprintf( stderr, "exit code %d\n", code );


  /* done */

  MPI_Finalize();

  return code;
}

#ifdef HAVE_DLSYM_AUTO_UNDERSCORE
#define dlsym dlsym_auto_underscore
void *dlsym_auto_underscore( void *handle, const char *symbol );
#endif

int happy( int argc, char **argv, MPI_Comm comm )
{
  inPut input       = { 0 };
  int   notFinished = 1;
  int   rank;

  /* dlopen */

  void   *handle         = dlopen( argv[1], RTLD_LAZY );
  init_f *initSearch     = (init_f *) dlsym( handle, "initSearch" );
  cond_f *conditionData  = (cond_f *) dlsym( handle, "conditionData" );
  appl_f *applySearch    = (appl_f *) dlsym( handle, "applySearch" );
  free_f *freeOutput     = (free_f *) dlsym( handle, "freeOutput" );
  fini_f *finalizeSearch = (fini_f *) dlsym( handle, "finalizeSearch" );

  /* get my rank */
  MPI_Comm_rank( comm, &rank );

  { /* initSearch */
    /* search master is process with rank = 0; there is no wrapper master */
    InitParams  initpar;
    int         initcode;
    char       *initmesg = NULL;
    initpar.argc = argc - 1;
    initpar.argv = argv + 1;
    initpar.argv[0] = "-filterparams";
    initpar.startTime = time( NULL );
    initpar.dataDuration = 0;
    initpar.realtimeRatio = 1;
    initpar.nodeClass = !rank;
    initcode = initSearch( &initmesg, &initpar );
    CHECK_CODE( "INIT", initcode, initmesg );
    if ( initmesg ) free( initmesg );
  }


  { /* conditionData */
    char  *condmesg = NULL;
    int    condcode = conditionData( &condmesg, &input, NULL );
    CHECK_CODE( "COND", condcode, condmesg );
    if ( condmesg ) free( condmesg );
  }


  while ( notFinished )
  {
    MPIapiAction mpiact = { 0 };
    SearchOutput output = { 0 };
    SearchParams params;
    params.comm   = &comm;
    params.action = &mpiact;

    { /* applySearch */
      char *applmesg = NULL;
      int   applcode = applySearch( &applmesg, &input, &output, &params );
      CHECK_CODE( "APPL", applcode, applmesg );
      if ( applmesg ) free( applmesg );
    }

    notFinished = output.notFinished;

    { /* freeOutput */
      char *freemesg = NULL;
      int   freecode = freeOutput( &freemesg, &output );
      CHECK_CODE( "FREE", freecode, freemesg );
      if ( freemesg ) free( freemesg );
    }
  }


  { /* finalizeSearch */
    char *finimesg = NULL;
    int   finicode = finalizeSearch( &finimesg );
    CHECK_CODE( "FINI", finicode, finimesg );
    if ( finimesg ) free( finimesg );
  }


  /* cleanup */

  dlclose( handle );

  return 0;
}
